local AddonName, a = ...
if a.BuildFail(50000) then return end
local L = a.Localize
local s = SpellFlashAddon
local x = s.UpdatedVariables

-- Implement a dummy TTD function if we do not get one from another module
if type(s.TimeToDie) ~= 'function'  then
	a.print(L["No TimeToDie function found, using a dummy function. Consider installing Pusikas' Spellflash: TTD."])
	s.TimeToDie = function(unit)
		return 10
	end
end

local HasteCDs = {
	49016--[[Unholy Frenzy]],
	80353--[[Time Warp]],
	2825--[[Bloodlust]],
	32182--[[Heroism]],
	90355--[[Ancient Hysteria]],
	26297--[[Berserking]],
}
function a.Hasted()
	return s.Buff(HasteCDs, "player")
end

-- Detect raid-wide haste buffs, so we know when to
-- use potions
local BloodlustSpells = {
	80353--[[Time Warp]],
	2825--[[Bloodlust]],
	32182--[[Heroism]],
	90355--[[Ancient Hysteria]],
}
function a.Bloodlust()
	return s.Buff(BloodlustSpells, "player")
end

a.MySelf = UnitGUID("player")

local LastAoETimestamp = 0
local CurrentTimestamp = 0
local LastAoETargetsHit = 0
local event, sourceGUID, sourceName, destGUID, spellId, spellName, amount, blocked, absorbed
local myShieldableDamage = {}
local myBlockableDamage = {}
local arrayIndex = 0
local tsIndex = 0
local myLastDamageTaken = time()
local i


local DamagingAoESpells = {
	[845--[[Cleave]]] = true,
	[6343--[[Thunder Clap]]] = true,
	[1680--[[Whirlwind]]] = true,
	[118000--[[Dragon Roar]]] = true,
	[52174--[[Heroic Leap]]] = true,
	[115767--[[Deep Wounds]]] = true,
}

local function clear()
	for i = 1, 60, 1 do
		myShieldableDamage[i] = 0
		myBlockableDamage[i] = 0
	end
end
clear()

function a.COMBAT_LOG_EVENT_UNFILTERED(event, ...)
	CurrentTimestamp, event, _, sourceGUID, sourceName, _, _, destGUID = ...
	-- turn the timestamp into an index for our arrays
	tsIndex = mod(floor(CurrentTimestamp * 10), 60) + 1
	
	if tsIndex > arrayIndex then
	-- the new index comes after the old index. we have to clear all array entries between them.
		for i = arrayIndex + 1, tsIndex, 1 do
			myShieldableDamage[i] = 0
			myBlockableDamage[i] = 0
		end
		arrayIndex = tsIndex
	elseif tsIndex < arrayIndex then
	-- the new index comes before the old index. we have to clear till the end of the array and then
	-- from the start up to the new index.
		for i = arrayIndex + 1, 60, 1 do
			myShieldableDamage[i] = 0
			myBlockableDamage[i] = 0
		end
		for i = 1, tsIndex, 1 do
			myShieldableDamage[i] = 0
			myBlockableDamage[i] = 0
		end
		arrayIndex = tsIndex
	end
		
	if sourceGUID == a.MySelf and destGUID ~= a.MySelf and (event == "SPELL_DAMAGE" or event == "SPELL_MISSED" or event == "SPELL_PERIODIC_DAMAGE") then
		spellId, spellName = select(12, ...)
		if DamagingAoESpells[spellId] then
			if CurrentTimestamp == LastAoETimestamp then
				LastAoETargetsHit = LastAoETargetsHit + 1
			else
				LastAoETargetsHit = 1
				LastAoETimestamp = CurrentTimestamp
			end
		elseif spellId == 12723--[[Sweeping Strikes]] then
			LastAoETargetsHit = 2
			LastAoETimestamp = CurrentTimestamp
		end
	elseif destGUID == a.MySelf and (string.find(event, "_DAMAGE") or string.find(event, "_MISSED"))then
		myLastDamageTaken = CurrentTimestamp
		if (event == "SPELL_DAMAGE" or event == "RANGE_DAMAGE") then
			spellId, _, _, amount, _, _, _, blocked, absorbed = select(12, ...)
			if blocked then
			-- we try to remember which attacks can be blocked
				if not BlockableAttacks[spellId] then
					BlockableAttacks[spellId] = sourceName
				end
				myBlockableDamage[arrayIndex] = myBlockableDamage[arrayIndex] + amount + (absorbed or 0) + blocked
			elseif BlockableAttacks[spellId] then
				myBlockableDamage[arrayIndex] = myBlockableDamage[arrayIndex] + amount + (absorbed or 0) + (blocked or 0)
			end
			-- we simply assume that all damage can be shielded
			myShieldableDamage[arrayIndex] = myShieldableDamage[arrayIndex] + amount + (absorbed or 0) + (blocked or 0)
		elseif (event == "SPELL_MISSED" or event == "RANGE_MISSED") then
			spellId, _, _, _, _, amount = select(12, ...)
			if BlockableAttacks[spellId] then
				myBlockableDamage[arrayIndex] = myBlockableDamage[arrayIndex] + (amount or 0)
			end
			myShieldableDamage[arrayIndex] = myShieldableDamage[arrayIndex] + (amount or 0)
		elseif destGUID == a.MySelf and (event == "SWING_DAMAGE") then
			amount, _, _, _, blocked, absorbed = select(12, ...)
			-- we assume that all swing damage can be blocked AND shielded
			myBlockableDamage[arrayIndex] = myBlockableDamage[arrayIndex] + amount + (absorbed or 0) + (blocked or 0)
			myShieldableDamage[arrayIndex] = myShieldableDamage[arrayIndex] + amount + (absorbed or 0) + (blocked or 0)
		elseif destGUID == a.MySelf and event == "SWING_MISSED" then
			_, _, amount = select(12, ...)
			myBlockableDamage[arrayIndex] = myBlockableDamage[arrayIndex] + (amount or 0)
			myShieldableDamage[arrayIndex] = myShieldableDamage[arrayIndex] + (amount or 0)
		elseif destGUID == a.MySelf and (event == "SPELL_PERIODIC_DAMAGE") then
			_, _, _, amount, _, _, _, _, absorbed = select(12, ...)
			myShieldableDamage[arrayIndex] = myShieldableDamage[arrayIndex] + amount + (absorbed or 0)
		end
	end
end
s.RegisterModuleEvent(AddonName, "COMBAT_LOG_EVENT_UNFILTERED", a.COMBAT_LOG_EVENT_UNFILTERED)

-- Check if we are in AoE mode
function a.AoE()
	if CurrentTimestamp < LastAoETimestamp + 3 then
		return LastAoETargetsHit
	else
		return 1
	end
end

function a.wouldHaveBlocked()
	if myLastDamageTaken < time() - 6 then
		return 0
	end
	local amount = 0
	local i, bc, cbc
	for i = 1, 60 do
		amount = amount + myBlockableDamage[i]
	end
	bc = GetBlockChance()
	if bc > 100 then bc = bc - 100 end
	cbc = 17.6 + GetCombatRating(CR_MASTERY) / 272.7
	amount = amount * 0.3 + amount * 0.3 * ((bc + cbc) / 100)
	return amount
end

function a.wouldHaveAbsorbed()
	if myLastDamageTaken < time() - 6 then
		return 0
	end
	local amount = 0
	local i, maxAbsorb
	for i = 1, 60 do
		amount = amount + myShieldableDamage[i]
	end
	local rage = UnitPower("player" , SPELL_POWER_RAGE)
	if rage < 20 then
		rage = 20
	end
	maxAbsorb = max(UnitAttackPower("player") * 180 / 100, UnitStat("player", 3) * 250 / 100) / 3 * min(rage, 60) / 20
	return min(amount, maxAbsorb)
end

function a.incomingDamage(duration)
	local searchIndex = mod(floor((CurrentTimestamp - duration) * 10), 60) + 1
	if not myShieldableDamage[searchIndex] then
		return 0
	end
	local ret = 0
	if searchIndex > arrayIndex then
		for i = arrayIndex, searchIndex, 1 do
			ret = ret + myShieldableDamage[i]
		end
	elseif searchIndex < arrayIndex then
		for i = arrayIndex, 60, 1 do
			ret = ret + myShieldableDamage[i]
		end
		for i = 1, searchIndex, 1 do
			ret = ret + myShieldableDamage[i]
		end
	end
	return ret
end

function a.getVengeance()
	local name,_,_,_,_,_,_,_,_,_,_,_,_,_,val1 = UnitAura("player", s.SpellName(132365--[[Vengeance]]), nil, "HELPFUL")
	if name then
		return val1 or 0
	end
	return 0
end

-- Scan for Readiness
x.CooldownReduction = 0
local count, iLevel, temp
local EEOG = {
	Trinket0Slot = {104993, 104744, 102298, 105242, 104495, 105491}, --[[Trinket 1]]
	Trinket1Slot = {104993, 104744, 102298, 105242, 104495, 105491}, --[[Trinket 2]]
}
x.Melee2T16 = false
x.Melee4T16 = false
local MeleeT16 = {
	ChestSlot = {99047, 99603, 99197, 99411},  --[[Chest]]
	HandsSlot = {99034, 99559, 99198, 99412}, --[[Gloves]]
	LegsSlot = {99035, 99560, 99199, 99413}, --[[Legs]]
	ShoulderSlot = {99036, 99561, 99200, 99414},  --[[Shoulder]]
	HeadSlot = {99046, 99602, 99206, 99418}, --[[Head]]
}
a.scanEquippedItems = function()
	count = 0
	for slot, list in pairs(EEOG) do
		if s.Equipped(list, slot) then
			count = count + 1
			_, _, _, iLevel = GetItemInfo(GetInventoryItemLink("player", GetInventorySlotInfo(slot)))
			temp = 1 / (1 + 17 * math.pow(1.15, ((iLevel - 463) / 15)))
		end
	end
	if count >= 1 then
		if temp ~= x.CooldownReduction then
			x.CooldownReduction = temp
			a.print(L["Found Evil Eye of Galakras. Increased cooldown recovery rate calculated."])
		end
	else
		if x.CooldownReduction ~= 1 then
			x.CooldownReduction = 1
			a.print(L["No Evil Eye of Galakras found."])
		end
	end
	count = 0
	for slot, list in pairs(MeleeT16) do
		if s.Equipped(list, slot) then
			count = count + 1
		end
	end
	if count < 2 then
		if x.Melee2T16 then
			x.Melee2T16 = false
			a.print(L["no 2T16 found"])
		end
		if x.Melee4T16 then
			x.Melee4T16 = false
			a.print(L["no 4T16 found"])
		end
	elseif count < 4 then
		if not x.Melee2T16 then
			x.Melee2T16 = true
			a.print(L["2T16 found"])
		end
		if x.Melee4T16 then
			x.Melee4T16 = false
			a.print(L["no 4T16 found"])
		end
	else
		if not x.Melee2T16 then
			x.Melee2T16 = true
			a.print(L["2T16 found"])
		end
		if not x.Melee4T16 then
			x.Melee4T16 = true
			a.print(L["4T16 found"])
		end
	end
end

s.RegisterModuleEvent(AddonName, "PLAYER_EQUIPMENT_CHANGED", a.scanEquippedItems)
a.scanEquippedItems()
